@@ -107,6 +107,7 @@
 #' results
 #' @seealso \code{\link{galeShapley.collegeAdmissions}}
 #' @aliases galeShapley
+#' @export
 galeShapley.marriageMarket = function(proposerUtils = NULL,
                    reviewerUtils = NULL,
                    proposerPref = NULL,
@@ -212,7 +213,11 @@ galeShapley = function(proposerUtils = NULL,
 #'   or C++ indexing (starting at 0).
 #' @param slots is the number of slots that each college has available. If this
 #'   is 1, then the algorithm is identical to
-#'   \code{\link{galeShapley.marriageMarket}}.
+#'   \code{\link{galeShapley.marriageMarket}}. \code{slots} can either be a
+#'   integer or a vector. If it is an integer, then all colleges have the
+#'   same number of slots. If it is a vector, it must have as many elements
+#'   as there are colleges where each element refers to the number of slots
+#'   at a particular college. 
 #' @param studentOptimal is \code{TRUE} if students apply to colleges. The
 #'   resulting match is student-optimal. \code{studentOptimal} is \code{FALSE}
 #'   if colleges apply to students. The resulting match is college-optimal.
@@ -230,10 +235,10 @@ galeShapley = function(proposerUtils = NULL,
 #'    college \code{j}. Slots that remain open show up as being matched to
 #'    student to \code{NA}.}
 #'    \item{\code{unmatched.students} is a vector that lists the remaining unmatched
-#'    students This vector will be empty whenever \code{n<=m*s}}.
+#'    students This vector will be empty when all students get matched.}
 #'    \item{\code{unmatched.colleges} is a vector that lists colleges with open
 #'    slots. If a college has multiple open slots, it will show up multiple
-#'    times. This vector will be empty whenever \code{m*s<=n}}.
+#'    times. This vector will be empty whenever all college slots get filled.}
 #'   }
 #' @examples
 #' ncolleges = 10
@@ -274,20 +279,36 @@ galeShapley = function(proposerUtils = NULL,
 #'                              slots = 2,
 #'                              studentOptimal = FALSE)
 #' results.collegeoptimal
+#' @export
 galeShapley.collegeAdmissions = function(studentUtils = NULL,
                     collegeUtils = NULL,
                     studentPref = NULL,
                     collegePref = NULL,
                     slots = 1,
                     studentOptimal = TRUE) {
+    
+    if(length(slots) > 1) {
+        if(!is.null((collegePref)) & (length(slots) != NCOL(collegePref)) | 
+           !is.null((collegeUtils)) & (length(slots) != NCOL(collegeUtils))) {
+            stop("slots must either be a scalar or have the same length as there are colleges.")
+        }
+    }
 
     if (studentOptimal) {
-
+        
         # validate the inputs
         args = galeShapley.validate(studentUtils, collegeUtils, studentPref, collegePref)
 
+        # number of students
+        number_of_students = NROW(args$reviewerUtils)
+        
         # number of colleges
-        number_of_colleges = NROW(args$reviewerUtils)
+        number_of_colleges = NCOL(args$reviewerUtils)
+        
+        # expand slots
+        if(length(slots)==1) { 
+            slots = rep(slots, number_of_colleges)    
+        }
 
         # expand cardinal utilities corresponding to the slot size
         proposerUtils = reprow(args$proposerUtils, slots)
@@ -299,7 +320,7 @@ galeShapley.collegeAdmissions = function(studentUtils = NULL,
         # use galeShapleyMatching to compute matching
         res = cpp_wrapper_galeshapley(proposerPref, reviewerUtils)
 
-        # number of workers
+        # number of students
         M = length(res$proposals)
 
         # number of positions
@@ -308,32 +329,44 @@ galeShapley.collegeAdmissions = function(studentUtils = NULL,
         # collect results
         res = c(res, list(
             "unmatched.students" = seq(from = 0, to = M - 1)[res$proposals == N] + 1,
-            "unmatched.colleges" = seq(from = 0, to = N - 1)[res$engagements == M] + 1
+            "unmatched.colleges" = rep(NA, length = sum(res$engagements == M))
         ))
+        unmatched.colleges = seq(from = 0, to = N - 1)[res$engagements == M] + 1
 
-        # collapse engagements (turn these into R indices by adding +1)
-        res$matched.colleges = matrix(res$engagements, ncol = slots, byrow = TRUE) + 1
+        # assemble results
+        res$matched.colleges = list()
+        
+        # map engagements back into slots
+        cumsum.slotsLower = cumsum(c(0, slots[-length(slots)]))+1
+        cumsum.slotsUpper = cumsum(slots)
+ 
+        for(jX in 1:number_of_colleges) {
+            # fill slots with student ids
+            res$matched.colleges[[jX]] = res$engagements[cumsum.slotsLower[jX]:cumsum.slotsUpper[jX]] + 1
+            # set vacant slots to NA
+            res$matched.colleges[[jX]][res$matched.colleges[[jX]] == (number_of_students + 1)] = NA
+            # unmatched colleges
+            res$unmatched.colleges[unmatched.colleges %in% (cumsum.slotsLower[jX]:cumsum.slotsUpper[jX])] = jX
+        }
+        # remove unused information from res
         res$engagements = NULL
-
-        # translate proposals into the id of the original firm (turn these into R indices by adding +1)
-        college.ids = reprow(matrix(seq(from = 0, to = number_of_colleges), ncol = 1), slots)
-        res$matched.students = matrix(college.ids[res$proposals + 1], ncol = 1) + 1
         res$proposals = NULL
 
-        # translate single colleges into the id of the original college (turn these into R indices by adding +1)
-        res$unmatched.colleges = college.ids[res$unmatched.colleges] + 1
-
-        # unmatched students / colleges are matched to NA
-        res$matched.colleges[res$matched.colleges == (M + 1)] = NA
-        res$matched.students[res$matched.students == (N/slots + 1)] = NA
-
     } else {
 
         # validate the inputs
         args = galeShapley.validate(collegeUtils, studentUtils, collegePref, studentPref)
 
+        # number of students
+        number_of_students = NROW(args$proposerUtils)
+        
         # number of colleges
-        number_of_colleges = NROW(args$proposerUtils)
+        number_of_colleges = NCOL(args$proposerUtils)
+        
+        # expand slots
+        if(length(slots)==1) { 
+            slots = rep(slots, number_of_colleges)    
+        }
 
         # expand cardinal utilities corresponding to the slot size
         proposerUtils = repcol(args$proposerUtils, slots)
@@ -345,34 +378,53 @@ galeShapley.collegeAdmissions = function(studentUtils = NULL,
         # use galeShapleyMatching to compute matching
         res = cpp_wrapper_galeshapley(as.matrix(proposerPref), as.matrix(reviewerUtils))
 
-        # number of firms
+        # number of slots
         M = length(res$proposals)
 
-        # number of workers
+        # number of students
         N = length(res$engagements)
 
         # collect results
         res = c(res, list(
-            "unmatched.colleges" = seq(from = 0, to = M - 1)[res$proposals == N] + 1,
+            "unmatched.colleges" = rep(NA, length = sum(res$proposals == N)),
             "unmatched.students" = seq(from = 0, to = N - 1)[res$engagements == M] + 1
         ))
-
-        # collapse proposals (turn these into R indices by adding +1)
-        res$matched.colleges = matrix(res$proposals, ncol = slots, byrow = TRUE) + 1
-        res$proposals = NULL
-
-        # translate engagements into the id of the original firm (turn these into R indices by adding +1)
-        college.ids = reprow(matrix(seq(from = 0, to = number_of_colleges), ncol = 1), slots)
-        res$matched.students = matrix(college.ids[res$engagements + 1], ncol = 1) + 1
+        unmatched.colleges = seq(from = 0, to = M - 1)[res$proposals == N] + 1
+        
+        # assemble results
+        res$matched.colleges = list()
+        
+        # map proposals back into slots
+        cumsum.slotsLower = cumsum(c(0, slots[-length(slots)]))+1
+        cumsum.slotsUpper = cumsum(slots)
+        
+        for(jX in 1:number_of_colleges) {
+            # fill slots with student ids
+            res$matched.colleges[[jX]] = res$proposals[cumsum.slotsLower[jX]:cumsum.slotsUpper[jX]] + 1
+            # set vacant slots to NA
+            res$matched.colleges[[jX]][res$matched.colleges[[jX]] == (number_of_students + 1)] = NA
+            # unmatched colleges
+            res$unmatched.colleges[unmatched.colleges %in% (cumsum.slotsLower[jX]:cumsum.slotsUpper[jX])] = jX
+        }
+        
+        # remove unused information from res
         res$engagements = NULL
-
-        # translate unmatched college slots into the id of the original college (turn these into R indices by adding +1)
-        res$unmatched.colleges = college.ids[res$unmatched.colleges] + 1
-
-        # unmatched students / colleges are matched to NA
-        res$matched.colleges[res$matched.colleges == (N + 1)] = NA
-        res$matched.students[res$matched.students == (M/slots + 1)] = NA
-
+        res$proposals = NULL
+        
+    }
+    
+    # make a vector with matched students
+    res$matched.students = matrix(NA, nrow = number_of_students, ncol=1)
+    for(jX in 1:number_of_colleges) {
+        res$matched.students[res$matched.colleges[[jX]]] = jX    
+    }
+    # remove proposals (all relevant information is stored in res$matched.students)
+    res$engagements = NULL
+    
+    # if all colleges have the same number of slots return matched.colleges as matrix 
+    # (otherwise it's a list)
+    if(all(slots == slots[1])) {
+        res$matched.colleges = matrix(unlist(res$matched.colleges), nrow = number_of_colleges, ncol = slots[1], byrow = TRUE)
     }
 
 
@@ -444,6 +496,7 @@ galeShapley.collegeAdmissions = function(studentUtils = NULL,
 #' # validate preferences when proposer-side is cardinal and reviewer-side is ordinal
 #' preferences = galeShapley.validate(proposerUtils = uM, reviewerPref = prefW)
 #' preferences
+#' @export
 galeShapley.validate = function(proposerUtils = NULL, reviewerUtils = NULL, proposerPref = NULL, reviewerPref = NULL) {
 
     if (!is.null(reviewerPref)) {
@@ -566,7 +619,12 @@ galeShapley.validate = function(proposerUtils = NULL, reviewerUtils = NULL, prop
 #'                            pref.validated$reviewerUtils,
 #'                            results$proposals,
 #'                            results$engagements)
+#' @export
 galeShapley.checkStability = function(proposerUtils, reviewerUtils, proposals, engagements) {
+    
+    if(is.list(proposals) | is.list(engagements)) {
+        stop("Proposals and engagements must be vectors/matrices.")  
+    }
 
     # replace NA for unmatched proposers (they are now matched to the number of reviewers + 1)
     proposals[is.na(proposals)] = NROW(proposerUtils) + 1
@@ -617,6 +675,7 @@ galeShapley.checkStability = function(proposerUtils, reviewerUtils, proposals, e
 #'                 2, 1, 2), nrow = 2, ncol = 3, byrow = TRUE)
 #' pref
 #' galeShapley.checkPreferences(pref)
+#' @export
 galeShapley.checkPreferences = function(pref) {
 
     # check if pref is using R instead of C++ indexing
